<?php

declare(strict_types=1);

namespace PrestaShop\Module\ConotoxiaPay\Install;

use Configuration;
use Db;
use Module;
use PrestaShop\Module\ConotoxiaPay\Configure\Configurator;
use PrestaShop\Module\ConotoxiaPay\Definitions;

/**
 * Class responsible for modifications needed during installation/uninstallation of the module.
 */
class Installer
{
    /**
     * Module's installation entry point.
     *
     * @param $module Module
     * @return bool
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    public function install($module)
    {
        if (!$this->registerHooks($module)) {
            return false;
        }
        if (!$this->installDatabase()) {
            return false;
        }
        if (!Configurator::addOrderWaitingStatus()) {
            return false;
        }
        return true;
    }

    /**
     * Module's uninstallation entry point.
     *
     * @return bool
     * @throws \PrestaShopException
     */
    public function uninstall()
    {
        return Configurator::removeOrderWaitingStatus()
            && Configuration::deleteByName(Definitions::CLIENT_ID)
            && Configuration::deleteByName(Definitions::CLIENT_SECRET)
            && Configuration::deleteByName(Definitions::POINT_OF_SALE)
            && Configuration::deleteByName(Definitions::PUBLIC_KEY)
            && Configuration::deleteByName(Definitions::PRIVATE_KEY)
            && Configuration::deleteByName(Definitions::SANDBOX)
            && Configuration::deleteByName(Definitions::ICON)
            && Configuration::deleteByName(Definitions::WAITING_STATUS)
            && $this->uninstallDatabase();
    }

    /**
     * Install the database modifications required for this module.
     *
     * @return bool
     */
    private function installDatabase()
    {
        $queries = [
            'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'conpay_payment` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `status` varchar(64) NOT NULL,
              `max_refund_achieved` tinyint(1) NOT NULL,
              `id_payment` varchar(64) NOT NULL,
              `id_order` int(11) NOT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;'
            ,
            'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'conpay_refund` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `date` datetime NOT NULL,
              `amount` decimal(10,2) NOT NULL DEFAULT \'0.00\',
              `id_currency` int(11) NOT NULL,
              `status` varchar(64) NOT NULL,
              `type` varchar(12) NOT NULL,
              `id_payment` varchar(64) NOT NULL,
              `id_refund` varchar(64) NOT NULL,
              `id_order` int(11) NOT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;'
        ];

        return $this->executeQueries($queries);
    }

    /**
     * Uninstall database modifications.
     *
     * @return bool
     */
    private function uninstallDatabase()
    {
        $queries = [
            'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'conpay_refund`',
            'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'conpay_payment`',
        ];

        return $this->executeQueries($queries);
    }

    /**
     * Register hooks for the module.
     *
     * @param Module $module
     *
     * @return bool
     */
    private function registerHooks($module)
    {
        $hooks = ['paymentReturn', 'payment', 'backOfficeHeader', 'displayAdminOrderLeft'];
        if (version_compare(_PS_VERSION_, '1.7', 'lt')) {
            $hooks[] = 'displayPaymentEU';
        } else {
            $hooks[] = 'paymentOptions';
        }

        $result = true;
        foreach ($hooks as $hook) {
            $result &= $module->registerHook($hook);
        }

        return $result;
    }

    /**
     * A helper that executes multiple database queries.
     *
     * @param array $queries
     *
     * @return bool
     */
    private function executeQueries($queries)
    {
        foreach ($queries as $query) {
            if (!Db::getInstance()->execute($query)) {
                return false;
            }
        }
        return true;
    }
}
